% Charles Swanson, cswanson@pppl.gov
% 2022/02/15

% This script compares a toolbox solution to the PROCESS generalized plane 
% strain solution. 
%
% Uses the Layered Axisymmetric Plane Strain Model and Solver Toolbox.
% You'll probably have to add it to your path variable. 

close all;clear all;

%-------------------------------------------------------------------------------
%% Set up input parameters

% Number of layers
N = 4;

r  =  [
  2.1019000000000001
  2.6128200000000001
  2.6228199999999999
  2.6902067648884511
  3.4493673863858731]'; % (m)

% Force in the Z direction
Fz = 2400080928.6658416; % (N)

% Layer thicknesses. dr(i+1) is thickness of layer i.
% dr(1) is innermost point. dr(1)=0 corresponds to plugged bore.
dr     = [0.82522 0.20225 0.93662 0.50233 0.23373 0.75613]; % (m)

% Innermost layer which still contributes to Fz
innerFzLayer = 3; % This layer and outward contribute to Fz

j = [0 0 0 13037585.452208769]; % (A/m^2)

% Young's moduli and Poisson's ratios
Ez   = [
  43360824543.045639     
  2500000000.0000000     
  205000000000.00000     
  113722445554.42221]'; % Axial Young's modulus (Pa)
Et   = [
  114438555400.00000     
  2500000000.0000000     
  205000000000.00000     
  35979923769.496368]'; % Transverse Young's modulus (Pa)
nut  = [
  0.29999999999999999     
  0.34000000000000002     
  0.29999999999999999     
  0.29999999999999999]'; % Transverse-transverse Poisson's ratio
nuzt = [
  0.29999999999999999     
  0.34000000000000002     
  0.29999999999999999     
  0.29999999999999999]'; % Axial-transverse Poisson's ratio
  
%-------------------------------------------------------------------------------
%% A couple small points of compatibility

dr = [r(1) r(2:end)-r(1:(end-1))];
dI = pi*j.*(r(2:end).^2 - r(1:(end-1)).^2);

%-------------------------------------------------------------------------------
%% Begin the analysis

% Set up the geometry and materials properties:

% Inter-layer spacing, r(1) is innermost surface, r(N+1) is outermost surface
% r(j) separates layer j-1 from layer j
% dr(1) = 0 corresponds to plugged bore
sumMat = 1:(N+1);
sumMat = sumMat>=sumMat';
r      = sum(sumMat.*dr',1);

% Stiffness form of the materials properties:
% nutz set by symmetry of compliance tensor
nutz    = nuzt .* Et ./ Ez;
[EzBar,EtBar,nuztBar,nutzBar,nutBar] = StiffnessElasticProperties(Ez,Et,nuzt,nut);

% General body force integrals
[~,fIntAVec,fIntBVec] = LorentzBodyForceInts(r(1:N),(1:N),r,dI);

% Generate each mInt(:,:,j), transfer matrix within one layer, such that
% (AIn(j);BIn(j);epsZOut;1;epsZIn) = mInt(:,:,j)*(AOut(j);BOut(j);epsZOut;1;epsZIn)
mInt = MInt(fIntAVec,fIntBVec,EtBar,5,4);

% Generate each mExt(:,:,j), transfer matrix for layer boundaries, such that 
% (AOut(j-1);BOut(j-1);epsZOut;1;epsZIn) = mExt(:,:,j)*(AIn(j);BIn(j);epsZOut;1;epsZIn)
strainLayers = 3 + 2*((1:N)<innerFzLayer);
mExt = MExt(r,EtBar,nutBar,nuztBar,5,strainLayers);

% Generate a total mTot matrix for ease of relating global boundaries, relating 
% AIn(j),BIn(j) to AOut(N),BOut(N)
mTot = MTot(mExt,mInt);

% Generate a mTension matrix which produces axial tensions of the layer subsets
mTension = MTension(r,EzBar,nutzBar,mTot,strainLayers);

% The boundary condition matrix, mBC*[AOut(N);BOut(N);epsZOut;1;epsZIn] = 0
% Radial stress at outer edge
mBC = MSigRRSq(r(end),nutBar(end),nuztBar(end),5,3);
% Radial stress at inner edge
mBC = [mBC; MSigRRSq(r(1),nutBar(1),nuztBar(1),mTot(:,:,1),5)];
% Axial force of both layer subsets
mBC = [mBC; mTension];

% The solution vector via matrix inversion
AVecSolution = SolveAugmented(mBC,[0;0;Fz;0],4);

fprintf('AVecSolution(1) = %.4g \n',AVecSolution(1));

%% PLOT OUTPUTS

% Plotting domain
drPlot = 1e-4;
rPlot  = r(1):drPlot:r(N+1);
%rPlot  = [rPlot r(N+1)];

[AVecPlot,lPlot] = LameVectorProfileFast(AVecSolution,rPlot,r,mTot,mExt,...
                   @LorentzBodyForceInts,dI,EtBar);

% Or the slower but more general version:
%[~,fIntAPlot,fIntBPlot] = LorentzBodyForceInts(rPlot,lPlot,r,dI);
%mIntPlot = MInt(fIntAPlot,fIntBPlot,EtBar(lPlot),5,4);
%[AVecPlot,lPlot] = LameVectorProfile(AVecSolution,rPlot,r,mTot,mExt,mIntPlot);

[epsRPlot,epsThetaPlot,epsZPlot,uPlot] = ...
                  StrainLameVector(AVecPlot,rPlot,strainLayers(lPlot));

% Stresses
mStiff = StiffnessMatrix(EzBar(lPlot),EtBar(lPlot),nuztBar(lPlot),nutzBar(lPlot),nutBar(lPlot));
sigPlot = MatrixMultiMult(mStiff,[epsRPlot;epsThetaPlot;epsZPlot]);

sigRPlot = sigPlot(1,:);sigThetaPlot = sigPlot(2,:);sigZPlot = sigPlot(3,:);

fprintf('Max radial stress is %.4g Pa \n',max(abs(sigRPlot)));
fprintf('Max azimut stress is %.4g Pa \n',max(abs(sigThetaPlot)));
fprintf('Max axial  stress is %.4g Pa \n',max(abs(sigZPlot)));

% Axial tension
dAPlot = 2*pi*rPlot.*drPlot;
dFz = dAPlot.*sigZPlot;
FzOut = sum(dFz(strainLayers(lPlot)==3));
FzIn  = sum(dFz(strainLayers(lPlot)==5));
fprintf('FzIn/Fz  = %.4g \n',FzIn/Fz);
fprintf('FzOut/Fz = %.4g \n',FzOut/Fz);

% PROCESS quantities
[rPROC,eRPROC,eTPROC,eZPROC,uPROC,sRPROC,sTPROC,sZPROC] = ...
              LoadPROCESS('../PROCESS/BuckedWedged_SIG_TF.DAT');

% Plot quantities
figure();
set(gcf,'position',[1 1 1.0*[1 1]].*get(gcf,'position'))
hold on;

% Displacement
subplot(2,2,1);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,uPlot,'linewidth',2);legendStr = [legendStr,'u'];
plot(rPROC,uPROC,'--','linewidth',2);legendStr = [legendStr,'uPROC'];
xlabel('r (m)');ylabel('u (m)');
grid on;
legend(legendStr);

% Strain
subplot(2,2,2);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,epsRPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_r'];
plot(rPROC,eRPROC,'--','linewidth',2);legendStr = [legendStr,'\epsilon_r PROC'];
plot(rPlot,epsThetaPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_\theta'];
plot(rPROC,eTPROC,'--','linewidth',2);legendStr = [legendStr,'\epsilon_\theta PROC'];
plot(rPlot,epsZPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_z'];
plot(rPROC,eZPROC,'--','linewidth',2);legendStr = [legendStr,'\epsilon_z PROC'];
xlabel('r (m)');ylabel('\epsilon');
grid on;
legend(legendStr);

% Stress
subplot(2,2,3);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,sigRPlot,'linewidth',2);legendStr = [legendStr,'\sigma_r'];
plot(rPROC,sRPROC,'--','linewidth',2);legendStr = [legendStr,'\sigma_r PROC'];
plot(rPlot,sigThetaPlot,'linewidth',2);legendStr = [legendStr,'\sigma_\theta'];
plot(rPROC,sTPROC,'--','linewidth',2);legendStr = [legendStr,'\sigma_\theta PROC'];
plot(rPlot,sigZPlot,'linewidth',2);legendStr = [legendStr,'\sigma_z'];
plot(rPROC,sZPROC,'--','linewidth',2);legendStr = [legendStr,'\sigma_z PROC'];
xlabel('r (m)');ylabel('\sigma (Pa)');
grid on;
legend(legendStr);
