% Charles Swanson, cswanson@pppl.gov
% 2022/02/16

% This script produces a radial stress and strain distribution to compare to 
% a simple sanity check
%
% This is the plugged external pressure case: Check that it acts like a scalar
% pressure in the transverse direction (sigR = sigT = -POut)
%
% Uses the Layered Axisymmetric Plane Strain Model and Solver Toolbox.
% You'll probably have to add it to your path variable. 

close all;clear all;

%-------------------------------------------------------------------------------
%% Set up input parameters

% Number of layers
N = 1;

% Layer thicknesses. dr(i+1) is thickness of layer i
% dr(1) is innermost point. dr(1)=0 corresponds to plugged bore.
dr     = [0 0.5]; % (m)

% Young's moduli and Poisson's ratios
E   = 2e11; % Transverse Young's modulus (Pa)
nu  = 0.30; % Transverse-transverse Poisson's ratio

% Axial strain
epsZ = 1e-3;

% Pressure outside
POut = 2e8; % (Pa)

%-------------------------------------------------------------------------------
%% Begin the analysis

% Set up the geometry and materials properties:
% Inter-layer spacing, r(1) is innermost surface, r(N+1) is outermost surface
% r(j) separates layer j-1 from layer j
% dr(1) = 0 corresponds to plugged bore
sumMat = 1:(N+1);
sumMat = sumMat>=sumMat';
r      = sum(sumMat.*dr',1);

% Stiffness form of the materials properties:
% nutz set by symmetry of compliance tensor
[ES,nuS] = StiffnessElasticProperties(E,nu);

% Generate a total mTot matrix for ease of relating global boundaries, relating 
% AIn(j),BIn(j) to AOut(N),BOut(N)
mTot = eye(4);

% The boundary condition matrix, mBC*[AOut(N);BOut(N);epsZ;1.0] = 0
% Radial stress at outer edge
mBC = MSigRRSq(r(end),nuS(end),[],4,3,ES);
% Radial stress at inner edge
mBC = [mBC; MSigRRSq(r(1),nuS(1),[],mTot(:,:,1),3,ES)];
% Axial strain
mBC = [mBC; [0 0 1 0]];

% The solution vector via matrix inversion
AVecSolution = SolveAugmented(mBC,[-POut*r(end)^2 0 epsZ]',4);

fprintf('AVecSolution(1) = %.4g \n',AVecSolution(1));
fprintf('AVecSolution(2) = %.4g \n',AVecSolution(2));

%% PLOT OUTPUTS

% Plotting domain
drPlot = 1e-4;
rPlot  = r(1):drPlot:r(N+1);

AVecPlot = repmat(AVecSolution,size(rPlot));

[epsRPlot,epsThetaPlot,epsZPlot,uPlot] = ...
                  StrainLameVector(AVecPlot,rPlot);

% Stresses
mStiff = StiffnessMatrix(ES,nuS);
sigPlot = MatrixMultiMult(mStiff,[epsRPlot;epsThetaPlot;epsZPlot]);

sigRPlot = sigPlot(1,:);sigThetaPlot = sigPlot(2,:);sigZPlot = sigPlot(3,:);

fprintf('Max radial stress is %.4g Pa \n',max(abs(sigRPlot)));
fprintf('Max azimut stress is %.4g Pa \n',max(abs(sigThetaPlot)));
fprintf('Max axial  stress is %.4g Pa \n',max(abs(sigZPlot)));

% Axial tension
dAPlot = 2*pi*rPlot.*drPlot;
dFz = dAPlot.*sigZPlot;
Fz = sum(dFz);
fprintf('Fz  = %.4g \n',Fz);

% Plot quantities
figure();
set(gcf,'position',[1 1 2*[1 1]].*get(gcf,'position'))
hold on;

% Displacement
subplot(2,2,1);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,uPlot,'linewidth',2);
legendStr = [legendStr,'u'];
xlabel('r (m)');ylabel('u');
grid on;
legend(legendStr);

% Strain
subplot(2,2,2);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,epsRPlot,'linewidth',2);
legendStr = [legendStr,'\epsilon_r'];
plot(rPlot,epsThetaPlot,'linewidth',2);
legendStr = [legendStr,'\epsilon_\theta'];
plot(rPlot,epsZPlot,'linewidth',2);
legendStr = [legendStr,'\epsilon_z'];
xlabel('r (m)');ylabel('\epsilon');
grid on;
legend(legendStr);

% Stress
subplot(2,2,3);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,sigRPlot,'linewidth',2);
legendStr = [legendStr,'\sigma_r'];
plot(rPlot,sigThetaPlot,'--','linewidth',2);
legendStr = [legendStr,'\sigma_\theta'];
plot(rPlot,sigZPlot,'linewidth',2);
legendStr = [legendStr,'\sigma_z'];
xlabel('r (m)');ylabel('\sigma (Pa)');
grid on;
legend(legendStr);
