% Charles Swanson, cswanson@pppl.gov
% 2022/05/19

% This script produces a radial stress and strain distribution to compare to 
% the Ansys benchmarks performed by Chirag Rana. 
%
% Geometry: Two polygonal shells of length 1 m: One with inner radius 0.33 m and outer radius 0.67 m, one with inner radius 0.67 m and outer radius 1 m.
% Materials: Inner layer: Isotropic Young's modulus of 2*10^11 Pa. Isotropic Poisson's ratio of 0.30. Outer layer: Isotropic Young's modulus of 1*10^11 Pa. Isotropic Poisson's ratio of 0.30
% Axial boundary conditions: A fixed axial displacement of 0 mm is applied to the upper surface of both layers.
% Radial boundary conditions: The radial outward faces of the outer layer is under a pressure of 2*10^8 Pa. The inner faces of the inner layer is under a pressure of 0 Pa.
% Radial force density: None.
%
% Uses the Layered Axisymmetric Plane Strain Model and Solver Toolbox.
% You'll probably have to add it to your path variable. 

close all;clear all;

%-------------------------------------------------------------------------------
%% Set up input parameters

% Number of layers
N = 2;

% Layer thicknesses. dr(i+1) is thickness of layer i
% dr(1) is innermost point. dr(1)=0 corresponds to plugged bore.
dr     = [0.33 0.34 0.33]; % (m)

% Young's moduli and Poisson's ratios
E   = [2e11 1e11]; % Isotropic Young's modulus (Pa)
nu  = [0.30 0.30]; % Isotropic Poisson's ratio

% Axial strain
epsZ = 0e-3;

% Pressure outside
POut = 2e8; % (Pa)

%boolCenter = true; % Center cord
boolCenter = false; % Corner cord

line = struct;
line(1).name = '20 sides';
line(1).fileAnsys = '20_side1.txt';
line(1).pointsToNeglect = 2;
line(1).radFac = 1/cos(pi/20);
line(1).linespec = 'r';
line(2).name = '16 sides';
line(2).fileAnsys = '16side_1.txt';
line(2).pointsToNeglect = 2;
line(2).radFac = 1/cos(pi/16);
line(2).linespec = 'r';
line(3).name = '12 sides';
line(3).fileAnsys = '12side_1.txt';
line(3).pointsToNeglect = 2;
line(3).radFac = 1/cos(pi/12);
line(3).linespec = 'r';
line(4).name = '10 sides';
line(4).fileAnsys = '10side1.txt';
line(4).pointsToNeglect = 2;
line(4).radFac = 1/cos(pi/10);
line(4).linespec = 'r';
%{
line(5).name = '6 sides';
line(5).fileAnsys = '6side1.txt';
line(5).pointsToNeglect = 3;
line(5).radFac = 1/cos(pi/6);
line(5).linespec = 'r';
%}

%-------------------------------------------------------------------------------
%% Begin the analysis

fTresca = @(x,y,z) max([abs(x-y);abs(x-z);abs(y-z)],[],1);

% Set up the geometry and materials properties:
% Inter-layer spacing, r(1) is innermost surface, r(N+1) is outermost surface
% r(j) separates layer j-1 from layer j
% dr(1) = 0 corresponds to plugged bore
sumMat = 1:(N+1);
sumMat = sumMat>=sumMat';
r      = sum(sumMat.*dr',1);

% Stiffness form of the materials properties:
% nutz set by symmetry of compliance tensor
[ES,nuS] = StiffnessElasticProperties(E,nu);

% Generate each mInt(:,:,j), transfer matrix within one layer, such that
% (AIn(j);BIn(j);epsZ;1) = mInt(:,:,j)*(AOut(j);BOut(j);epsZ;1)
mInt = repmat(eye(4),[1 1 N]);

% Generate each mExt(:,:,j), transfer matrix for layer boundaries, such that 
% (AOut(j-1);BOut(j-1);epsZ;1) = mExt(:,:,j)*(AIn(j);BIn(j);epsZ;1)
mExt = MExt(r,ES,nuS,[],4);

% Generate a total mTot matrix for ease of relating global boundaries, relating 
% AIn(j),BIn(j) to AOut(N),BOut(N)
mTot = MTot(mExt,mInt);

% The boundary condition matrix, mBC*[AOut(N);BOut(N);epsZOut;1;] = 0
% Radial stress at outer edge
mBC = MSigRRSq(r(end),nuS(end),[],4,3,ES(end));
% Radial stress at inner edge
mBC = [mBC; MSigRRSq(r(1),nuS(1),[],mTot(:,:,1),3,ES(1))];
% Axial strain
mBC = [mBC; [0 0 1 0]];

% The solution vector via matrix inversion
AVecSolution = SolveAugmented(mBC,[-POut*r(end)^2 0 epsZ]',4);

fprintf('AVecSolution(1) = %.4g \n',AVecSolution(1));
fprintf('AVecSolution(2) = %.4g \n',AVecSolution(2));

%% PLOT OUTPUTS

% Plotting domain
drPlot = 1e-4;
rPlot  = r(1):drPlot:r(N+1);
%lPlot = interp1(r,1:numel(r),rPlot,'previous');
%lPlot = min(lPlot,N);

[AVecPlot,lPlot] = LameVectorProfileFast(AVecSolution,rPlot,r,mTot,mExt,...
                   @NullBodyForceInts,0,ES);

[epsRPlot,epsThetaPlot,epsZPlot,uPlot] = ...
                  StrainLameVector(AVecPlot,rPlot);

% Stresses
mStiff = StiffnessMatrix(ES,nuS);
sigPlot = MatrixMultiMult(mStiff(:,:,lPlot),[epsRPlot;epsThetaPlot;epsZPlot]);

sigRPlot = sigPlot(1,:);sigThetaPlot = sigPlot(2,:);sigZPlot = sigPlot(3,:);
sigTrescaPlot = fTresca(sigRPlot,sigThetaPlot,sigZPlot);

fprintf('Max radial stress is %.4g Pa \n',max(abs(sigRPlot)));
fprintf('Max azimut stress is %.4g Pa \n',max(abs(sigThetaPlot)));
fprintf('Max axial  stress is %.4g Pa \n',max(abs(sigZPlot)));
fprintf('Max Tresc  stress is %.4g Pa \n',max(abs(sigTrescaPlot)));

% Axial tension
dAPlot = 2*pi*rPlot.*drPlot;
dFz = dAPlot.*sigZPlot;
Fz = sum(dFz);
fprintf('Fz  = %.4g \n',Fz);


for lineInd = 1:numel(line)
% Ansys quantities
  rCenter = r(1)*line(lineInd).radFac;

  [rAnsys,sigAnsys] = LoadChiragTXT(line(lineInd).fileAnsys,rCenter);

%  rAnsys = rAnsys(line(lineInd).pointsToNeglect:end);
%  rAnsys = rAnsys;%*line(lineInd).radFac;

%  sigAnsys = sigAnsys(line(lineInd).pointsToNeglect:end);
  tn = line(lineInd).pointsToNeglect;
  neglectInds = 1:tn;
  neglectDeriv = (sigAnsys(tn+2)-sigAnsys(tn+1)) / (rAnsys(tn+2)-rAnsys(tn+1));
  sigAnsys(neglectInds) = sigAnsys(tn+1) + neglectDeriv*(rAnsys(neglectInds)-rAnsys(tn+1));
  
  line(lineInd).rAnsys   = rAnsys;
  line(lineInd).sigAnsys = sigAnsys;
  fprintf('Title %s, min azimuthal stress %.5g Pa \n',line(lineInd).name,min(sigAnsys));
  fprintf('Discrepancy %.4g pcent \n',(min(sigAnsys)-min(sigThetaPlot))/min(sigThetaPlot)*100);

end

%% Last plots
% Plot quantities
figure();
set(gcf,'position',[1 1 0.7*[1 1]].*get(gcf,'position'))
hold on;

% Azimuthal Stress
%subplot(1,2,1);
legendStr = {};
plot(rPlot,sigThetaPlot/1e6,'b--','linewidth',2);legendStr = [legendStr,'0D cylinder'];
hold on;
for lineInd = 1:numel(line)
% Ansys quantities
%  plot(line(lineInd).rAnsys,line(lineInd).sigAnsys/1e6,line(lineInd).linespec,'linewidth',1);legendStr = [legendStr,['Ansys ' line(lineInd).name]];
  plot(line(lineInd).rAnsys,line(lineInd).sigAnsys/1e6,'linewidth',1);legendStr = [legendStr,['Ansys ' line(lineInd).name]];
end
xlabel('r (m)');ylabel('\sigma_\theta (MPa)');
grid on;
legend(legendStr,'location','southeast');


