% Charles Swanson, cswanson@pppl.gov
% 2022/02/16

% This script produces a radial stress and strain distribution to compare to 
% the Ansys benchmarks performed by Chirag Rana. 
%
% This is Script 03:
% Geometry: A cylindrical shell of length 1 m, inner radius 0.25 m, and outer radius 0.5 m.
% Materials: Isotropic Young's modulus of 2*10^11 Pa. Isotropic Poisson's ratio of 0.30
% Axial boundary conditions: A fixed axial displacement of 1 mm is applied to the upper surface of both layers.
% Radial boundary conditions: The inner and outer faces are under 0 Pa.
% Radial force density: The layer has a uniform axial current density of 47.6 MA/m^2 (same magnetic pressure as earlier boundary condition). 
% Outputs: Radial, azimuthal, and axial stress and strain, along a line from the center of the cylinder and progressing radially outward.
%
% Check this: Each volume element has in inward volumetric body force of (1.422*10^9 N/m^3 * (r/1m) - 3.555*10^8 N/m^3 * (1m/r))
%
% Uses the Layered Axisymmetric Plane Strain Model and Solver Toolbox.
% You'll probably have to add it to your path variable. 

close all;clear all;

%-------------------------------------------------------------------------------
%% Set up input parameters

% Number of layers
N = 1;

% Layer inner radii. r(N+1) is outer radius.
r     = [0.25 0.5]; % (m)

% Young's moduli and Poisson's ratios
E   = 2e11; % Isotropic Young's modulus (Pa)
nu  = 0.3; % Isotropic Poisson's ratio

% Axial strain
epsZ = 1e-3*0;

% Pressure outside
POut = 2e8; % (Pa)

fileAnsys = 'S03_Lorentz.csv';

%-------------------------------------------------------------------------------
%% A couple small points of compatibility

dr = [r(1) r(2:end)-r(1:(end-1))];

mu0 = 4*pi*1e-7;
B = sqrt(2*mu0*POut);
dI = 2*pi*r(end)*B/mu0;

% Current density
j       = dI./pi./(r(end)^2-r(1)^2);
% Factors that multiply r linearly and reciprocally in force density
fLinFac = mu0/2 * j.^2;
fRecFac = - mu0/2 * j^2 *r(1)^2;
fExpression = @(r) fLinFac*r + fRecFac./r;

fprintf('The layer carries %.4g A of current \n',dI);
fprintf('The layer carries %.4g A/m^2 of current density \n',j);
fprintf('The body force expression is %d N/m^3 * (r/1m) + %d N/m^3 * (1m/r) \n',fLinFac,fRecFac);

%-------------------------------------------------------------------------------
%% Begin the analysis

% Set up the geometry and materials properties:
% Inter-layer spacing, r(1) is innermost surface, r(N+1) is outermost surface
% r(j) separates layer j-1 from layer j
% dr(1) = 0 corresponds to plugged bore
sumMat = 1:(N+1);
sumMat = sumMat>=sumMat';
r      = sum(sumMat.*dr',1);

% Stiffness form of the materials properties:
% nutz set by symmetry of compliance tensor
[ES,nuS] = StiffnessElasticProperties(E,nu);

% General body force integrals
[~,fIntAVec,fIntBVec] = LorentzBodyForceInts(r(1:N),(1:N),r,dI);

% Generate each mInt(:,:,j), transfer matrix within one layer, such that
% (AIn(j);BIn(j);epsZ;1) = mInt(:,:,j)*(AOut(j);BOut(j);epsZ;1)
mInt = MInt(fIntAVec,fIntBVec,ES,4);

% Generate each mExt(:,:,j), transfer matrix for layer boundaries, such that 
% (AOut(j-1);BOut(j-1);epsZ;1) = mExt(:,:,j)*(AIn(j);BIn(j);epsZ;1)
mExt = eye(4);

% Generate a total mTot matrix for ease of relating global boundaries, relating 
% AIn(j),BIn(j) to AOut(N),BOut(N)
mTot = MTot(mExt,mInt);

% The boundary condition matrix, mBC*[AOut(N);BOut(N);epsZOut;1epsZIn] = 0
% Radial stress at outer edge
mBC = MSigRRSq(r(end),nuS(end),[],4,3,ES(end));
% Radial stress at inner edge
mBC = [mBC; MSigRRSq(r(1),nuS(1),[],mTot(:,:,1),3,ES(1))];
% Axial strain
mBC = [mBC; [0 0 1 0]];

% The solution vector via matrix inversion
AVecSolution = SolveAugmented(mBC,[0 0 epsZ]',4);

fprintf('AVecSolution(1) = %.4g \n',AVecSolution(1));
fprintf('AVecSolution(2) = %.4g \n',AVecSolution(2));

%% PLOT OUTPUTS

% Plotting domain
drPlot = 1e-4;
rPlot  = r(1):drPlot:r(N+1);

[AVecPlot,lPlot] = LameVectorProfileFast(AVecSolution,rPlot,r,mTot,mExt,...
                   @LorentzBodyForceInts,dI,ES);

fPlot = LorentzBodyForceInts(rPlot,lPlot,r,dI);
fExpr = fExpression(rPlot);

[epsRPlot,epsThetaPlot,epsZPlot,uPlot] = ...
                  StrainLameVector(AVecPlot,rPlot);

% Stresses
mStiff = StiffnessMatrix(ES,nuS);
sigPlot = MatrixMultiMult(mStiff(:,:,lPlot),[epsRPlot;epsThetaPlot;epsZPlot]);

sigRPlot = sigPlot(1,:);sigThetaPlot = sigPlot(2,:);sigZPlot = sigPlot(3,:);

fprintf('Max radial stress is %.4g Pa \n',max(abs(sigRPlot)));
fprintf('Max azimut stress is %.4g Pa \n',max(abs(sigThetaPlot)));
fprintf('Max axial  stress is %.4g Pa \n',max(abs(sigZPlot)));

% Axial tension
dAPlot = 2*pi*rPlot.*drPlot;
dFz = dAPlot.*sigZPlot;
Fz = sum(dFz);
fprintf('Fz  = %.4g \n',Fz);

% Ansys quantities
[rAnsys,sRAnsys,sTAnsys,sZAnsys] = LoadChiragCSV(fileAnsys,r(1));

% Plot quantities
figure();
set(gcf,'position',[1 1 2.0*[1 1]].*get(gcf,'position'))
hold on;

% Displacement
subplot(2,2,1);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,uPlot,'linewidth',2);legendStr = [legendStr,'u'];
xlabel('r (m)');ylabel('u');
grid on;
legend(legendStr);

% Strain
subplot(2,2,2);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,epsRPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_r'];
plot(rPlot,epsThetaPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_\theta'];
plot(rPlot,epsZPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_z'];
xlabel('r (m)');ylabel('\epsilon');
grid on;
legend(legendStr);

% Stress
subplot(2,2,3);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,sigRPlot,'linewidth',2);legendStr = [legendStr,'\sigma_r'];
plot(rAnsys,sRAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_r Ansys'];
plot(rPlot,sigThetaPlot,'linewidth',2);legendStr = [legendStr,'\sigma_\theta'];
plot(rAnsys,sTAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_\theta Ansys'];
plot(rPlot,sigZPlot,'linewidth',2);legendStr = [legendStr,'\sigma_z'];
plot(rAnsys,sZAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_z Ansys'];
xlabel('r (m)');ylabel('\sigma (Pa)');
grid on;
legend(legendStr);

%% Second plot
% Plot quantities
figure();
set(gcf,'position',[1 1 0.7*[1 1]].*get(gcf,'position'))
hold on;

% Stress
legendStr = {};
plot(rPlot,sigRPlot/1e6,'b','linewidth',1);legendStr = [legendStr,'\sigma_r'];
hold on;
plot(rAnsys,sRAnsys/1e6,'b--','linewidth',3);legendStr = [legendStr,'\sigma_r Ansys'];
plot(rPlot,sigThetaPlot/1e6,'r','linewidth',1);legendStr = [legendStr,'\sigma_\theta'];
plot(rAnsys,sTAnsys/1e6,'r--','linewidth',3);legendStr = [legendStr,'\sigma_\theta Ansys'];
plot(rPlot,sigZPlot/1e6,'k','linewidth',1);legendStr = [legendStr,'\sigma_z'];
plot(rAnsys,sZAnsys/1e6,'k--','linewidth',3);legendStr = [legendStr,'\sigma_z Ansys'];
xlabel('r (m)');ylabel('\sigma (MPa)');
grid on;
legend(legendStr,'location','southeast');