% Charles Swanson, cswanson@pppl.gov
% 2022/02/16

% This script produces a radial stress and strain distribution to compare to 
% the Ansys benchmarks performed by Chirag Rana. 
%
% This is Script 05:
% Geometry: Two cylindrical shells of length 1 m: One with inner radius 0.33 m and outer radius 0.67 m, one with inner radius 0.67 m and outer radius 1 m.
% Materials: Inner layer: Isotropic Young's modulus of 2*10^11 Pa. Isotropic Poisson's ratio of 0.30. Outer layer: Transverse-isotropic Young's modulus: 2*10^11 Pa in the axial direction, 1*10^11 Pa in the transverse direction. Transverse-isotropic Poisson's ratio: 0.30 in the axial-transverse direction, 0.15 in the transverse-axial direction, and 0.5 in the transverse-transverse direction.
% Axial boundary conditions: A fixed axial displacement of 1 mm is applied to the upper surface of both layers.
% Radial boundary conditions: The radial outward face of the outer layer is under a pressure of 2*10^8 Pa. The inner face of the inner layer is under a pressure of 0 Pa.
% Radial force density: None.
%
% Uses the Layered Axisymmetric Plane Strain Model and Solver Toolbox.
% You'll probably have to add it to your path variable. 

close all;clear all;

%-------------------------------------------------------------------------------
%% Set up input parameters

% Number of layers
N = 2;

% Layer thicknesses. dr(i+1) is thickness of layer i
% dr(1) is innermost point. dr(1)=0 corresponds to plugged bore.
dr     = [0.33 0.34 0.33]; % (m)

% Young's moduli and Poisson's ratios
Ez   = [2e11 2e11]; % Axial Young's modulus (Pa)
Et   = [2e11 1e11]; % Transverse Young's modulus (Pa)
nut  = [0.3 0.5]; % Transverse-transverse Poisson's ratio
nuzt = [0.3 0.3]; % Axial-transverse Poisson's ratio

% Axial strain
epsZ = 1e-3*0;

% Pressure outside
POut = 2e8; % (Pa)

fileAnsys = 'S05_Anisotropic.csv';

%-------------------------------------------------------------------------------
%% Begin the analysis

% Set up the geometry and materials properties:
% Inter-layer spacing, r(1) is innermost surface, r(N+1) is outermost surface
% r(j) separates layer j-1 from layer j
% dr(1) = 0 corresponds to plugged bore
sumMat = 1:(N+1);
sumMat = sumMat>=sumMat';
r      = sum(sumMat.*dr',1);

% Stiffness form of the materials properties:
% nutz set by symmetry of compliance tensor
[ESz,ESt,nuSzt,nuStz,nuSt] = StiffnessElasticProperties(Ez,Et,nuzt,nut);

% Generate each mInt(:,:,j), transfer matrix within one layer, such that
% (AIn(j);BIn(j);epsZ;1) = mInt(:,:,j)*(AOut(j);BOut(j);epsZ;1)
mInt = repmat(eye(4),[1 1 N]);

% Generate each mExt(:,:,j), transfer matrix for layer boundaries, such that 
% (AOut(j-1);BOut(j-1);epsZ;1) = mExt(:,:,j)*(AIn(j);BIn(j);epsZ;1)
mExt = MExt(r,ESt,nuSt,nuStz,4);

% Generate a total mTot matrix for ease of relating global boundaries, relating 
% AIn(j),BIn(j) to AOut(N),BOut(N)
mTot = MTot(mExt,mInt);

% The boundary condition matrix, mBC*[AOut(N);BOut(N);epsZOut;1;] = 0
% Radial stress at outer edge
mBC = MSigRRSq(r(end),nuSt(end),nuSzt(end),4,3,ESt(end));
% Radial stress at inner edge
mBC = [mBC; MSigRRSq(r(1),nuSt(1),nuSzt(1),mTot(:,:,1),3,ESt(1))];
% Axial strain
mBC = [mBC; [0 0 1 0]];

% The solution vector via matrix inversion
AVecSolution = SolveAugmented(mBC,[-POut*r(end)^2 0 epsZ]',4);

fprintf('AVecSolution(1) = %.4g \n',AVecSolution(1));
fprintf('AVecSolution(2) = %.4g \n',AVecSolution(2));

%% PLOT OUTPUTS

% Plotting domain
drPlot = 1e-4;
rPlot  = r(1):drPlot:r(N+1);
%lPlot = interp1(r,1:numel(r),rPlot,'previous');
%lPlot = min(lPlot,N);

[AVecPlot,lPlot] = LameVectorProfileFast(AVecSolution,rPlot,r,mTot,mExt,...
                   @NullBodyForceInts,0,ESt);

[epsRPlot,epsThetaPlot,epsZPlot,uPlot] = ...
                  StrainLameVector(AVecPlot,rPlot);

% Stresses
mStiff = StiffnessMatrix(ESz,ESt,nuSzt,nuStz,nuSt);
sigPlot = MatrixMultiMult(mStiff(:,:,lPlot),[epsRPlot;epsThetaPlot;epsZPlot]);

sigRPlot = sigPlot(1,:);sigThetaPlot = sigPlot(2,:);sigZPlot = sigPlot(3,:);

fprintf('Max radial stress is %.4g Pa \n',max(abs(sigRPlot)));
fprintf('Max azimut stress is %.4g Pa \n',max(abs(sigThetaPlot)));
fprintf('Max axial  stress is %.4g Pa \n',max(abs(sigZPlot)));

% Axial tension
dAPlot = 2*pi*rPlot.*drPlot;
dFz = dAPlot.*sigZPlot;
Fz = sum(dFz);
fprintf('Fz  = %.4g \n',Fz);

% Ansys quantities
[rAnsys,sRAnsys,sTAnsys,sZAnsys] = LoadChiragCSV(fileAnsys,r(1));

% Plot quantities
figure();
set(gcf,'position',[1 1 2.0*[1 1]].*get(gcf,'position'))
hold on;

% Displacement
subplot(2,2,1);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,uPlot,'linewidth',2);legendStr = [legendStr,'u'];
xlabel('r (m)');ylabel('u');
grid on;
legend(legendStr);

% Strain
subplot(2,2,2);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,epsRPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_r'];
plot(rPlot,epsThetaPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_\theta'];
plot(rPlot,epsZPlot,'linewidth',2);legendStr = [legendStr,'\epsilon_z'];
xlabel('r (m)');ylabel('\epsilon');
grid on;
legend(legendStr);

% Stress
subplot(2,2,3);
legendStr = {};
plot(r,zeros(size(r)),'+','linewidth',3);legendStr = [legendStr,'Boundaries'];
hold on;
plot(rPlot,sigRPlot,'linewidth',2);legendStr = [legendStr,'\sigma_r'];
plot(rAnsys,sRAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_r Ansys'];
plot(rPlot,sigThetaPlot,'linewidth',2);legendStr = [legendStr,'\sigma_\theta'];
plot(rAnsys,sTAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_\theta Ansys'];
plot(rPlot,sigZPlot,'linewidth',2);legendStr = [legendStr,'\sigma_z'];
plot(rAnsys,sZAnsys,'+','linewidth',1);legendStr = [legendStr,'\sigma_z Ansys'];
xlabel('r (m)');ylabel('\sigma (Pa)');
grid on;
legend(legendStr,'location','southeast');

%% Second plot
% Plot quantities
figure();
set(gcf,'position',[1 1 0.7*[1 1]].*get(gcf,'position'))
hold on;

% Stress
legendStr = {};
plot(rPlot,sigRPlot/1e6,'b','linewidth',1);legendStr = [legendStr,'\sigma_r'];
hold on;
plot(rAnsys,sRAnsys/1e6,'b--','linewidth',3);legendStr = [legendStr,'\sigma_r Ansys'];
plot(rPlot,sigThetaPlot/1e6,'r','linewidth',1);legendStr = [legendStr,'\sigma_\theta'];
plot(rAnsys,sTAnsys/1e6,'r--','linewidth',3);legendStr = [legendStr,'\sigma_\theta Ansys'];
plot(rPlot,sigZPlot/1e6,'k','linewidth',1);legendStr = [legendStr,'\sigma_z'];
plot(rAnsys,sZAnsys/1e6,'k--','linewidth',3);legendStr = [legendStr,'\sigma_z Ansys'];
xlabel('r (m)');ylabel('\sigma (MPa)');
grid on;
legend(legendStr,'location','southeast');