function [f] = InboardTensionMoment(rInner,rOuter)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/03/22
%
% Uses the force and moment balance to determine the fraction of the vertical
% separating force on the inboard leg of the TF coil. However, compression 
% rings and slip joints can change this. This is a first cut only. 
%
%-------------------------------------------------------------------------------
% Usage:
% [f] = InboardTensionMoment(rInner,rOuter)
%-------------------------------------------------------------------------------
% Inputs:
% rInner(:,:)      Radius of the inboard leg of the TF coil (m)
% rOuter(:,:)      Radius of the outboard leg of the TF coil (m)
%-------------------------------------------------------------------------------
% Outputs:
% f(:,)            Fraction of the total vertical separating force taken up by
%                  the inboard leg tension.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  dr = 1e-2;
  rInner = 1;
  rOuter = 1+(dr:dr:30);
  InboardTensionMoment(rInner,rOuter);
  f = [];
  return
end

%-------------------------------------------------------------------------------

r = rOuter./rInner;
b = (1+r.*log(r)-r)./(r-1-log(r));
f = b./(b+1);

%-------------------------------------------------------------------------------
% Default output
if (nargout==0)
  if (max(numel(rInner),numel(rOuter))==1)
    fprintf('Vertical separating force inboard fraction is %.4g \n',f);
  else
    figure();
    plot(r,f);
    grid on;
    xlabel('Radius ratio');ylabel('Inboard fraction of vertical separating force');
  end  
end

end