function [AVec,layers] = LameVectorProfile(AVecEdge,r,rLayers,mTot,mExt,mInt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/15
%
% Finds the radial profile of the vector of Lame thick cylinder parameters, 
% given the Lame vector at the edge. The Lame vector elements must start with
% A and B such that 
% u = A*r + B/r
% The other elements of the vector include axial strain
%
%-------------------------------------------------------------------------------
% Usage:
% [AVec,layers] = LameVectorProfile(AVecEdge,r,rLayers,mTot,mExt,mInt)
%-------------------------------------------------------------------------------
% Inputs:
% AVecEdge(m,1)    The vector of Lame thick cylinder parameters at the outer 
%                  edge of the member.
% r(1,:)           The radii of the different points at which to evaluate
%                  the Lame vector (m)
% rLayers(1,N+1)   The inner radius of each layer (m). rLayers(N+1) is the outer 
%                  edge of the outermost layer.
% mTot(m,m,N)      Each mTot(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the Lame vector at the inner edge of layer i.
% mExt(m,m,N)      Each mExt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the inner edge of layer i to produce the Lame 
%                  vector at the outer edge of layer i-1.
% mInt(m,m,:)      Each mInt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of layer i to produce the Lame 
%                  vector at the test point r
%-------------------------------------------------------------------------------
% Outputs:
% AVec(m,:)        The Lame vector at each rLayers point
% layers(1,:)      The layer index within each rLayers point resides
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  AVecOut = [];
  return
end
%-------------------------------------------------------------------------------
% Defaults
%-------------------------------------------------------------------------------

N      = numel(rLayers)-1;
layers = interp1(rLayers,1:numel(rLayers),r,'previous');
layers = min(layers,N);

AVecIn       = MatrixMultiMult(mTot,AVecEdge);
AVecOut      = MatrixMultiMult(mExt(:,:,2:N),AVecIn(:,2:N));
AVecOut(:,N) = AVecEdge;

%[~,fIntAPlot,fIntBPlot] = BodyForce(r,layers,rLayers,BodyForceInputs);
%mInt = MInt(fIntAPlot,fIntBPlot,ESt(layers),5,4);
AVec = MatrixMultiMult(mInt,AVecOut(:,layers));

end