function [AVec,layers] = LameVectorProfileFast(AVecEdge,r,rLayers,mTot,mExt,...
                            BodyForce,BodyForceInputs,ESt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/15
%
% Finds the radial profile of the vector of Lame thick cylinder parameters, 
% given the Lame vector at the edge. The Lame vector elements must start with
% A and B such that 
% u = A*r + B/r
% The other elements of the vector include axial strain
% Specifically this is the "fast" version which takes advantage of the known
% form of mInt to achieve a speedup. If the form of mInt changes, you'll need
% to use LameVectorProfile rather than LameVectorProfileFast.
%
%-------------------------------------------------------------------------------
% Usage:
% [AVec,layers] = LameVectorProfileFast(AVecEdge,r,rLayers,mTot,mExt,...
%                    BodyForce,BodyForceInputs,ESt)
%-------------------------------------------------------------------------------
% Inputs:
% AVecEdge(m,1)    The vector of Lame thick cylinder parameters at the outer 
%                  edge of the member.
% r(1,:)           The radii of the different points at which to evaluate
%                  the Lame vector (m)
% rLayers(1,N+1)   The inner radius of each layer (m). rLayers(N+1) is the outer 
%                  edge of the outermost layer.
% mTot(m,m,N)      Each mTot(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the Lame vector at the inner edge of layer i.
% mExt(m,m,N)      Each mExt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the inner edge of layer i to produce the Lame 
%                  vector at the outer edge of layer i-1.
% BodyForce(@)     Function handle to the function that produces the body force
%                  integrals, such as UniformBodyForceInts or
%                  LorentzBodyForceInts.
% BodyForceInputs  The inputs to that function
% ESt(1,N)         The transverse Young's modulus in stiffness form of each 
%                  layer (Pa). If isotropic, the isotropic Young's modulus in 
%                  stiffness form.
%-------------------------------------------------------------------------------
% Outputs:
% AVec(m,:)        The Lame vector at each rLayers point
% layers(1,:)      The layer index within each rLayers point resides
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  AVecOut = [];
  return
end
%-------------------------------------------------------------------------------
% Defaults
%-------------------------------------------------------------------------------

N      = numel(rLayers)-1;
layers = interp1(rLayers,1:numel(rLayers),r,'previous');
layers = min(layers,N);

AVecIn       = MatrixMultiMult(mTot,AVecEdge);
AVecOut      = MatrixMultiMult(mExt(:,:,2:N),AVecIn(:,2:N));
if (isempty(AVecOut))
  AVecOut = AVecEdge;
else
  AVecOut(:,N) = AVecEdge;
end

AVec = repmat(AVecEdge,[1 numel(r)]);

[~,fIntAPlot,fIntBPlot] = BodyForce(r,layers,rLayers,BodyForceInputs);
%mIntPlot = MInt(fIntAPlot,fIntBPlot,ESt(layers),5,4);
%AVec = MatrixMultiMult(mIntPlot,AVecOut(:,layers));
AVec(1,:) = AVecOut(1,layers) - 1/2./ESt(layers).*fIntAPlot;
AVec(2,:) = AVecOut(2,layers) + 1/2./ESt(layers).*fIntBPlot;

end