function [r,eR,eT,eZ,u,sR,sT,sZ] = LoadPROCESS(fName)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/16
%
% Loads in the stresses and strains from a PROCESS SIG_TF.DAT output file.
%
%-------------------------------------------------------------------------------
% Usage:
% [r,eR,eT,eZ,u,sR,sT,sZ] = LoadPROCESS(fName)
%-------------------------------------------------------------------------------
% Inputs:
% fName(1,:)       Path name to the relevant SIG_TF.DAT output file
%-------------------------------------------------------------------------------
% Outputs:
% r(1,:)           Radii of the reported points (m)
% eR(1,:)          Radial strain of the reported points
% eT(1,:)          Azimuthal strain of the reported points
% eZ(1,:)          Axial strain of the reported points
% u(1,:)           Radial displacement of the reported points (m)
% sR(1,:)          Radial stress of the reported points (Pa). "Smeared" strain
%                  using lumped element properties.
% sT(1,:)          Azimuthal stress of the reported points (Pa). "Smeared" 
%                  strain using lumped element properties.
% sZ(1,:)          Axial stress of the reported points (Pa). "Smeared" strain
%                  using lumped element properties.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  [r,~,~,~,u] = LoadPROCESS(...
  '~/SystemsStudies/CylinderLame/PlaneStrain/MATLAB/EarlyWork/AssessEffects/BuckedWedged_SIG_TF.DAT');
  figure();plot(r,u);grid on;
  xlabel('Radius (m)');ylabel('Radial displacement (m)');
  r = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
%-------------------------------------------------------------------------------

if ~exist(fName)
  fprintf('LoadPROCESS reports that this file name does not exist: \n');
  fprintf('  %s \n',fName);
  r = [];
  return
end

fileID = fopen(fName);
fileCells = {};
labelChars = 24;
line = fgetl(fileID);
line = fgetl(fileID);
for loopInd = 1:20
  line = fgetl(fileID);
  if numel(line)>labelChars
    fileCells = [fileCells; {line(1:labelChars), str2num(line(labelChars+1:end))}];
  end
end

r  = fileCells{1,2};
eR = fileCells{12,2};
eT = fileCells{13,2};
eZ = fileCells{14,2};
u  = fileCells{11,2}*1e-3;
sR = fileCells{5,2}*1e6;
sT = fileCells{6,2}*1e6;
sZ = fileCells{7,2}*1e6;

end