function [f,fIntA,fIntB] = LorentzBodyForceInts(r,layer,rLayers,dI)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/09
%
% Finds the relevant integrals of the volumetric radial body force to transform
% the thick cylinder Lame parameters A,B between two different radii
% Assumes each layer has a uniform axial current density, and body force 
% results from Ampere's Law and Lorentz's Law.
% Specifically, used to compute A(r),B(r) given A,B at the next layer transition
% outward of r.
%
% Described in Section 13 of the derivation doc
%
%-------------------------------------------------------------------------------
% Usage:
% [f,fIntA,fIntB] = LorentzBodyForceInts(r,layer,rLayers,dI)
%-------------------------------------------------------------------------------
% Inputs:
% r(1,:)           Radii of points at which to evaluate the force integrals (m)
% layer(1,:)       The layer number of each r(1,:) point
% rLayers(1,N+1)   The inner radii of each layer. rLayers(N+1) is the outer 
%                  radius of the outermost layer, layer N.
% dI(1,N)          Current carried by each layer (A)
%-------------------------------------------------------------------------------
% Outputs:
% f                Volumetric radial body force at each r point (N/m^3)
% fIntA            Integral of f between r and the next outer rLayer point. 
%                  This is used to transform the Lame A parameter at the outer
%                  point to the A parameter at r. (N/m^2)
% fIntB            Integral of f*r^2 between r and the next outer rLayer point. 
%                  This is used to transform the Lame B parameter at the outer
%                  point to the B parameter at r. (N/m)
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  dr = 1e-2;
  rLayers = [0.25 0.5 0.75];
  dI      = [2 1];
  r = rLayers(1):dr:rLayers(end);
  layer(r<rLayers(2))  = 1;
  layer(r>=rLayers(2)) = 2;
  [f,fIntA,fIntB] = LorentzBodyForceInts(r,layer,rLayers,dI);
  figure();
  subplot(2,2,1);
  plot(r,f);xlabel('r (m)');ylabel('f (N/m^3)');grid on;
  subplot(2,2,2);
  plot(r,fIntA);xlabel('r (m)');ylabel('fIntA (N/m^2)');grid on;
  subplot(2,2,3);
  plot(r,fIntB);xlabel('r (m)');ylabel('fIntB (N)');grid on;
  f = [];
  return
end

%-------------------------------------------------------------------------------

N = numel(dI);
% Permeability of free space
mu0    = 4*pi*1e-7;
% Current enclosed within the inner radius of each layer
sumMat = 1:N;
sumMat = sumMat>sumMat';
IEnc   = sum(sumMat.*dI',1);
% Current density of each layer
j       = dI./pi./(rLayers(2:end).^2-rLayers(1:(end-1)).^2);
% Factors that multiply r linearly and reciprocally in force density
fLinFac = mu0/2 * j.^2;
fRecFac = mu0/2 * (j.*IEnc/pi - j.^2.*rLayers(1:N).^2);

% Lorentz body force f
% Body force
f = fLinFac(layer).*r + fRecFac(layer)./r;
% A integral, integral of f: (including correction for if r(1)==0)
fIntA = 1/2*fLinFac(layer).*(rLayers(layer+1).^2-r.^2) + ...
  fRecFac(layer).*log(rLayers(layer+1)./(r+(fRecFac(layer)==0)));
% B integral, integral of f*r^2:
fIntB = 1/4*fLinFac(layer).*(rLayers(layer+1).^4-r.^4) + ... 
  1/2*fRecFac(layer).*(rLayers(layer+1).^2-r.^2);

end