function [mExt] = MExt(r,ESt,nuSt,nuSzt,mSize,strainLayers)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/10
%
% Finds the matrix which transforms the vector of thick cylinder Lame parameters
% A,B between two different radii, specifically used to compute A,B at the 
% outer surface of the next uniform layer inward,  given A,B at the outer
% surface of some layer.
% For example:
% [AOut(i-1);BOut(i-1);epsZ;1.0] = mExt(:,:,i)*[AIn(i);BIn(i);epsZ;1.0]
% They differ because they have different transverse stiffnesses and different
% Poisson ratios. The inter-layer boundary conditions are enforced.
% Elements 1 and 2 of the Lame vector are A and B respectively.
% This function supports multiple subsets of layers under multiple different 
% axial strains (epsZ), whose element numbers in the solution vector are 
% configurable via strainLayers
%
% Described in Section 6 of the derivation doc
% With transverse-isotropic materials properties from Section 12
% With axial decoupling of inner layers from Section 15
%
%-------------------------------------------------------------------------------
% Usage:
% [mExt] = MExt(r,ESt,nuSt,nuSzt,mSize,strainLayers)
%-------------------------------------------------------------------------------
% Inputs:
% r(1,N+1)         The inner radii of each layer. rLayers(N+1) is the outer 
%                  radius of the outermost layer, layer N.
% ESt(1,N)         Transverse Young's modulus in the stiffness form of each 
%                  layer (Pa). If isotropic, the isotropic Young's modulus in 
%                  stiffness form.
% nuSt(1,N)        Transverse-transverse Poisson's ratio in the stiffness form
%                  of each layer. If isotropic, the isotropic Poisson's ratio in 
%                  stiffness form.
% nuSzt(1,N)       Axial-transverse Poisson's ratio in the stiffness form
%                  of each layer. If isotropic, this can be empty.
% mSize            The number of elements in the Lame vector. Minimum is 3:
%                  one for A, one for B, and one for the in/homogeneous 
%                  coordinate 1.0. May be more if there are axial strains.
% strainLayers(1,N)
%                  For each layer, the element number of the solution vector 
%                  that corresponds to the axial strain of that layer.
%                  0 is no axial strain in the solution vector.
%                  For example, if every layer were under the axial 
%                  strain given in the 4th element of the solution vector, 
%                  strainLayers would be [4 4 4 4 ...]. Default is zeros if 
%                  mSize == 3, and all in element 3 if mSize > 3. 
%-------------------------------------------------------------------------------
% Outputs:
% mExt(mSize,mSize,N)
%                  Each mExt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the inner edge of layer i to produce the Lame 
%                  vector at the outer edge of layer i-1.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  mExt = MExt([1 2 3],[1 2 3],[3 2 1])
  mExt = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
if (nargin<=3)
  nuSzt = [];
end
if (isempty(nuSzt))
  nuSzt = nuSt;
end
if (nargin<=4)
  mSize = [];
end
if (isempty(mSize))
  mSize = 3;
end
if (nargin<=5)
  if (mSize==3)
    strainLayers = zeros(size(ESt));
  else
    strainLayers = 3*ones(size(ESt));
  end
end
%-------------------------------------------------------------------------------

N = numel(ESt);
% Make an mExt with an extra column to throw away, for strainLayers=0 layers
mExt = zeros(mSize,mSize+1,N);
% Use strainLayers mSize+1 for 0; store them in the extra mExt column
strainLayers = mod(strainLayers-1,mSize+1)+1;

% mExt needs to have dimension 3 be the layer dimension
r     = permute(r,[1 3 2]);
ESt   = permute(ESt,[1 3 2]);
nuSt  = permute(nuSt,[1 3 2]);
nuSzt = permute(nuSzt,[1 3 2]);

% Contribution of the inner layer
mExt(1,1,2:N) = mExt(1,1,2:N) + 1/2*ESt(1:N-1).*(1-nuSt(1:N-1));
mExt(1,2,2:N) = mExt(1,2,2:N) + 1/2*ESt(1:N-1)./r(2:N).^2.*(1-nuSt(1:N-1));
for layer = 1:N-1
  mExt(1,strainLayers(layer),layer+1) = mExt(1,strainLayers(layer),layer+1) ...
                                        - 1/2*ESt(layer)*nuSzt(layer);
end

% Contribution of the outer layer
mExt(1,1,2:N) = mExt(1,1,2:N) + 1/2*ESt(2:N).*(1+nuSt(2:N));
mExt(1,2,2:N) = mExt(1,2,2:N) - 1/2*ESt(2:N)./r(2:N).^2.*(1-nuSt(2:N));
for layer = 2:N
  mExt(1,strainLayers(layer),layer) = mExt(1,strainLayers(layer),layer) ...
                                      + 1/2*ESt(layer)*nuSzt(layer);
end

% Divide out the Young's modulus
mExt(1,:,2:N) = mExt(1,:,2:N)./ESt(1:N-1);

% Fill in Row 2 from Row 1
mExt(2,1,2:N) = r(2:N).^2.*(1-mExt(1,1,2:N));
mExt(2,2,2:N) = (1-r(2:N).^2.*mExt(1,2,2:N));
mExt(2,3:mSize,2:N) = -r(2:N).^2.*mExt(1,3:mSize,2:N);

% Fill in elements for those elements that don't change
for layer = 2:N
  mExt(3:mSize,3:mSize,layer) = (3:mSize)==(3:mSize)';
end

% Throw away the strainLayers=0 column
mExt = mExt(:,1:mSize,:);

end