function [mInt] = MInt(fIntA,fIntB,ESt,mSize,coordHomo)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/10
%
% Finds the matrix which transforms the vector of thick cylinder Lame parameters
% (A,B) between two different radii, specifically used to compute (A,B) at the 
% inner surface of a uniform layer, given (A,B) at the outer surface.
% They differ because of some radial body force density f.
% For example, [AIn(i);BIn(i);1.0] = mInt(:,:,i)*[AOut(i);BOut(i);1.0]
% Elements 1 and 2 of the Lame vector are A and B respectively. The 
% in/homogeneous element 1.0 is the last element by default, but can be set to
% any other element. 
%
% Described in Section 5 of the derivation doc
% With transverse-isotropic materials properties from Section 12
% With axial decoupling of inner layers from Section 15
%
%-------------------------------------------------------------------------------
% Usage:
% [mInt] = MInt(fIntA,fIntB,ESt,mSize,coordHomo)
%-------------------------------------------------------------------------------
% Inputs:
% fIntA(1,N)       A integral of each layer, produced by LorentzBodyForceInts, 
%                  UniformBodyForceInts, or equivalent
% fIntB(1,N)       B integral of each layer, produced by LorentzBodyForceInts, 
%                  UniformBodyForceInts, or equivalent
% ESt(1,N)         Transverse Young's modulus in the stiffness form of each 
%                  layer (Pa). If isotropic, the isotropic Young's modulus in 
%                  stiffness form.
% mSize            The number of elements in the Lame vector. Minimum is 3:
%                  one for A, one for B, and one for the in/homogeneous 
%                  coordinate 1.0. May be more if there are axial strains.
% coordHomo        The element number of the in/homogeneous coordinate 1.0 in 
%                  the Lame vector. Most sources put it at the end.
%-------------------------------------------------------------------------------
% Outputs:
% mInt(mSize,mSize,N)
%                  Each mInt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of layer i to produce the Lame 
%                  vector at the inner edge of layer i.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  mInt = MInt([1 2 3],[1 2 3],[3 2 1])
  mInt = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
if (nargin<=3)
  mSize = [];
end
if (isempty(mSize))
  mSize = 3;
end
if (nargin<=4)
  coordHomo = mSize;
end
%-------------------------------------------------------------------------------

N = numel(ESt);
mInt = repmat(eye(mSize),[1 1 N]);

mInt(1,coordHomo,:) = -1/2./ESt.*fIntA;
mInt(2,coordHomo,:) = 1/2./ESt.*fIntB;

end