function [mSigRRSq] = MSigRRSq(r,nuSt,nuSzt,arg4,strainLayers,ESt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/14
%
% Finds the matrix which computes the radial stress (sigR) times radius squared
% (sigR*r^2) from the vector of thick cylinder Lame parameters (A,B,etc.). This 
% is useful because it can be a boundary condition for the inner boundary even
% if there is no bore (plugged). If an mTot matrix is given, mSigRRSq multiplies
% the Lame vector at the outer edge of the outer layer, (AOut(N),BOut(N),etc.)
% For example:
% [sigR*r^2] = mSigRRSq*[A;B;epsZ;1.0]
% Or if mTot is given:
% [sigR*r^2] = mSigRRSq*[AOut(N);BOut(N);epsZ;1.0]
% Elements 1 and 2 of the Lame vector are A and B respectively. The other 
% elements must include a in/homogeneous coordinate 1.0, and may include a set
% of axial strains epsZ. Please configure those via the MExt and MInt functions.
%
% Described in Section 5 of the derivation doc
% With the r^2 factor described in Section 9
% With transverse-isotropic materials properties from Section 12
% With axial decoupling of inner layers from Section 15
%
%-------------------------------------------------------------------------------
% Usage:
% [mSigRRSq] = MSigRRSq(r,nuSt,nuSzt,mSize,strainLayers,ESt)
% [mSigRRSq] = MSigRRSq(r,nuSt,nuSzt,mTot,strainLayers,ESt)
%-------------------------------------------------------------------------------
% Inputs:
% r(1,:)           The radii at which to evaluate sigRRSq. 
% nuSt(1,:)        Transverse-transverse Poisson's ratio in stiffness form. If 
%                  isotropic, the isotropic Poisson's ratio in stiffness form.
% nuSzt (1,:)      Axial-transverse Poisson's ratio in stiffness form. If 
%                  isotropic, can be empty.
% mSize            The number of elements in the Lame vector. Minimum is 3:
%                  one for A, one for B, and one for the in/homogeneous 
%                  coordinate 1.0. May be more if there are axial strains.
% mTot(mSize,mSize,:)
%                  Each mTot(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the Lame vector at the inner edge of layer i. Include if 
%                  you want mSigRRSq to multiply the outer Lame vector, not the
%                  local Lame vector.
% strainLayers(1,:)
%                  The element number of the Lame vector that corresponds to the
%                  axial strain. Default is element 3.
% ESt(1,:)         Transverse Young's modulus in stiffness form at each point. 
%                  Default: 1. 
%-------------------------------------------------------------------------------
% Outputs:
% mSigRRSq(:,mSize)
%                  Each mSigRRSq(i,mSize) is the row which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the radial stress *r^2 at point i.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  [mSigRRSq] = MSigRRSq(1,0.3,[],4,3)
  mSigRRSq = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
if (nargin<=2)
  nuSzt = [];
end
if (isempty(nuSzt))
  nuSzt = nuSt;
end
if (numel(arg4)>1)
  mTot = arg4;
  mSize = size(mTot,2);
else
  mTot = eye(arg4);
  mSize = arg4;
end
if (nargin<=4)
  strainLayers = [];
end
if (isempty(strainLayers))
  strainLayers = 3*ones(1,numel(r));
end
if (nargin<=5)
  ESt = ones(1,numel(r));
end
%-------------------------------------------------------------------------------

mSigRRSq = zeros(numel(r),mSize);

for point = 1:numel(r)
  mSigRRSq(point,1) = (1+nuSt(point))*r(point)^2;
  mSigRRSq(point,2) = -1+nuSt(point);
  mSigRRSq(point,strainLayers(point)) = nuSzt(point)*r(point)^2;
  mSigRRSq(point,:) = ESt(point)*mSigRRSq(point,:)*mTot(:,:,point);
end

end