function [mTension] = MTension(r,ESz,nuStz,mTot,strainLayers)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/14
%
% Finds the matrix which computes the axial tension of the member, or of each 
% subset of layers. Each row of mTension multiplies the vector of thick 
% cylinder Lame parameters (A,B,etc.) to produce the axial tension of the layers
% corresponding to that row. 
% For example:
% [Fz1;Fz2] = mTension(:,:)*[AOut(N);BOut(N);epsZ1;epsZ2;1.0]
% Elements 1 and 2 of the Lame vector are A and B respectively. The other 
% elements must include a in/homogeneous coordinate 1.0, and may include a set
% of axial strains epsZ. Please configure those via the MExt and MInt functions.
%
% Described in Section 8 of the derivation doc
% With transverse-isotropic materials properties from Section 12
% With axial decoupling of inner layers from Section 15
%
%-------------------------------------------------------------------------------
% Usage:
% [mTension] = MTension(r,ESz,nuStz,mTot,strainLayers)
%-------------------------------------------------------------------------------
% Inputs:
% r(1,N+1)         The inner radii of each layer. r(N+1) is the outer radius
%                  of the outermost layer, layer N.
% ESz(1,N)         Axial Young's modulus in stiffness form of each layer (Pa).
%                  If Young's modulus is isotropic, this is the isotropic 
%                  Young's modulus in stiffnes form.
% nuStz(1,N)       Transverse-axial Poisson's ratio in stiffness form of each 
%                  layer. If isotropic, the isotropic Poisson's ratio in 
%                  stiffness form.
% mTot(mSize,mSize,N)
%                  Each mTot(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the Lame vector at the inner edge of layer i.
% strainLayers(1,N)
%                  For each layer, the element number of the solution vector 
%                  that corresponds to the axial strain of that layer.
%                  For example, if every layer were under the axial 
%                  strain given in the 4th element of the solution vector, 
%                  strainLayers would be [4 4 4 4 ...]. Default is all in 
%                  element 3.
%-------------------------------------------------------------------------------
% Outputs:
% mTension(:,mSize)
%                  Each mTension(i,mSize) is the row which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the axial tension of layer subset i, where the layer 
%                  subsets are configured via strainLayers. The order of the 
%                  rows is the same order as the order of the axial strains 
%                  in the solution vector. For example, [A;B;epsZ1;epsZ2,1.0]
%                  would yield [Fz1;Fz2] where Fz1 is the axial tension of those
%                  layers under axial strain epsZ1, and same of 2.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  mTension = MTension([0.5 1],200e9,0.3,eye(3))
  mTension = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
if (nargin<=4)
  strainLayers = [];
end
if (isempty(strainLayers))
  strainLayers = 3*ones(1,numel(r)-1);
end
%-------------------------------------------------------------------------------

N         = numel(r)-1;
mSize     = size(mTot,1);
strainEls = unique(strainLayers);

mTension = zeros(numel(strainEls),mSize);

%% Axial stiffness contribution

% Hooke's constant for each layer
dkHooke    = pi*ESz.*(r(2:end).^2-r(1:N).^2);
for ind = 1:numel(strainEls);
  strainEl = strainEls(ind);
  mTension(ind,strainEl) = sum(dkHooke(strainLayers == strainEl));
end

%% Poisson effect contribution  

rmRow = [r'.^2 ones(numel(r),1) zeros(numel(r),mSize-2)];
for layer = 1:N
  rmRow(layer,:) = rmRow(layer,:)*mTot(:,:,layer);
end
for ind = 1:numel(strainEls);
  layers = 1:N;
  layers = layers(strainLayers == strainEls(ind));
  mTension(ind,:) = mTension(ind,:) + ...
                    sum(2*pi*ESz(layers)'.*nuStz(layers)'.* ...
                    (rmRow(layers+1,:)-rmRow(layers,:)),1);
end

end