function [mTot] = MTot(mExt,mInt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/14
%
% Finds the matrix which transforms the vector of thick cylinder Lame parameters
% A,B. This one finds the Lame vector at the inner edge of each layer, given
% the Lame vector at the outer edge of the outermost layer. 
% For example:
% [Ain(i);BIn(i);epsZ;1.0] = mTot(:,:,i)*[AOut(N);BOut(N);epsZ;1.0]
% Elements 1 and 2 of the Lame vector are A and B respectively. The other 
% elements must include a in/homogeneous coordinate 1.0, and may include a set
% of axial strains epsZ. Please configure those via the MTot and MInt functions.
%
% Described in Section 7 of the derivation doc
% With axial decoupling of inner layers from Section 15
%
%-------------------------------------------------------------------------------
% Usage:
% [mTot] = MTot(mExt,mInt)
%-------------------------------------------------------------------------------
% Inputs:
% mExt(mSize,mSize,N)
%                  Each mExt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the inner edge of layer i to produce the Lame 
%                  vector at the outer edge of layer i-1.
% mInt(mSize,mSize,N)
%                  Each mInt(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of layer i to produce the Lame 
%                  vector at the inner edge of layer i.
%-------------------------------------------------------------------------------
% Outputs:
% mTot(mSize,mSize,N)
%                  Each mTot(:,:,i) is the matrix which multiplies the Lame
%                  vector at the outer edge of the outermost layer to produce
%                  the Lame vector at the inner edge of layer i.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  mInt = repmat([1 0.1;-0.1 1],[1 1 10]);
  mExt = repmat(inv(mInt(:,:,1)),[1 1 10]);
  mTot = MTot(mExt,mInt)
  mTot = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults

%-------------------------------------------------------------------------------

N     = size(mInt,3);
mSize = size(mInt,1);
mTot  = zeros(mSize,mSize,N);
mTot(:,:,N) = mInt(:,:,N)*((1:mSize)==(1:mSize)');
for layer = (N-1):-1:1
  mTot(:,:,layer) = mInt(:,:,layer)*mExt(:,:,layer+1)*mTot(:,:,layer+1);
end

end