function [vOut] = MatrixMultiMult(m,vIn)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/15
%
% Performs many matrix multiplications of one or many vectors concurrently.
% Matrices are packed into dimension 3. Rearranges them into a cell array and
% uses cellfun to run them all concurrently.
% Basically it does this:
% vOut(:,i) = m(:,:,i)*vIn
% or
% vOut(:,i) = m(:,:,i)*vIn(:,i)
% but quite fast
%
%-------------------------------------------------------------------------------
% Usage:
% [vOut] = MatrixMultiMult(m,vIn)
%-------------------------------------------------------------------------------
% Inputs:
% m(p,q,N)         The matrixes that multiply the vector, packed into dimension 
%                  3, so that vOut(:,i) = m(:,:,i)*vIn(:,i)
% vIn(q,N)         The vector that m multiplies. Can have size N in dimension 2,
%                  in which case each m multiplies a different column, or size 1
%                  in dimension 2, in which case m all multiply this vector.
%-------------------------------------------------------------------------------
% Outputs:
% vOut(p,N)        The vector that results when m are multiplied by vIn
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  theta = permute(0:pi/100:2*pi,[1 3 2]);
  m = [cos(theta) sin(theta);-sin(theta) cos(theta)];
  vIn = [1;0];
  vOut = MatrixMultiMult(m,vIn);
  plot(theta(:),vOut(1,:));
  vOut = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults

%-------------------------------------------------------------------------------
if size(vIn,2)==1
  vIn = repmat(vIn,[1 size(m,3)]);
end

mCell = permute(mat2cell(m,size(m,1),size(m,2),ones(1,size(m,3))),[1 3 2]);
vCell = mat2cell(vIn,size(vIn,1),ones(1,size(vIn,2)));
vOut  = cell2mat(cellfun(@mtimes,mCell,vCell,'uniformoutput',false));

end