function [vSolution] = SolveAugmented(mLHS,vRHS,coordHomo)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/14
%
% Solves an equation of the form:
% mLHS*vSolution = vRHS
% where some element of vSolution is an in/homogeneous element, 1.0. 
% It's just the backslash operator, rearranged a bit. 
% You may notice I write "in/homogeneous coordinate" a lot. That's because
% the nomenclature confuses me. When augmented coordinate systems are discussed
% in the literature, this is called the "homogeneous coordinate." But when 
% otherwise linear differential equations are solved, the solution including the
% augmented term is called the "inhomogeneous solution."
%
%-------------------------------------------------------------------------------
% Usage:
% [vSolution] = SolveAugmented(mLHS,vRHS,coordHomo)
%-------------------------------------------------------------------------------
% Inputs:
% mLHS(m-1,m)      The matrix that multiplies the solution vector in the 
%                  equation to solve. 
% vRHS(m,1)        The vector produced by mLHS when it multiplies vSolution.
%                  Default: Zeros.
% coordHomo        The element number of vRHS and vSolution which are 1.0, the
%                  in/homogeneous element. 
%                  Default: The last coordinate.
%-------------------------------------------------------------------------------
% Outputs:
% vSolution(m,1)   The vector that multiplies mLHS to produce vRHS.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  mLHS = rand(3,4)
  [vSolution] = SolveAugmented(mLHS,(1:3)')
  mLHS*vSolution
  vSolution = [];
  return
end

%-------------------------------------------------------------------------------
% Defaults
if (nargin<=1)
  vRHS = [];
end
if (isempty(vRHS))
  vRHS = zeros(size(mLHS,1),1);
end
if (nargin<=2)
  coordHomo = size(mLHS,2);
end
%-------------------------------------------------------------------------------
m = size(mLHS,2);
vSolution = ones(m,1);

inds = 1:m;
inds = inds(inds~=coordHomo);

vSolution(inds) = mLHS(:,inds)\(vRHS-mLHS(:,coordHomo));

end