function [out1,out2,out3,out4,out5] = StiffnessElasticProperties(arg1,arg2,arg3,arg4)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/09
%
% Inverts the 3D Hooke's Law properties from compliance form into stiffness
% form. Analytic for isotropic and transverse-isotropic. 
%
% Described in Section 3 of the derivation doc
% With transverse-isotropic materials properties from Section 12
%
%-------------------------------------------------------------------------------
% Usage:
% [ES,nuS] = StiffnessElasticProperties(E,nu)
% [ESz,ESt,nuSzt,nuStz,nuSt] = StiffnessElasticProperties(Ez,Et,nuzt,nut)
%-------------------------------------------------------------------------------
% Inputs:
% Set 1:
% E                Isotropic Young's modulus (Pa)
% nu               Isotropic Poisson's ratio
% Set 2:
% Ez               Axial Young's modulus (Pa)
% Et               Transverse Young's modulus (Pa)
% nuzt             Poisson's ratio in the axial-transverse direction
% nut              Poisson's ratio in the transverse-transverse direction
%-------------------------------------------------------------------------------
% Outputs:
% ES               Young's modulus in stiffness form (Pa)
% nuS              Poisson's ratio in stiffness form
% ESz              Axial Young's modulus in stiffness form (Pa)
% ESt              Transverse Young's modulus in stiffness form (Pa)
% nuSzt            Poisson's ratio in the axial-transverse direction in 
%                  stiffness form
% nuStz            Poisson's ratio in the transverse-axial direction in 
%                  stiffness form
% nuSt             Poisson's ratio in the transverse-transverse direction in 
%                  stiffness form
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  E  = 200e9; % Pa, steel
  nu = 0.3; % steel
  [ES,nuS] = StiffnessElasticProperties(E,nu);
  fprintf('Stiffness Young''s modulus ES  is %.4g Pa \n',ES);
  fprintf('Stiffness Poisson''s ratio nuS is %.4g \n',nuS);
  return
end

%-------------------------------------------------------------------------------
if (nargin<=2) % Isotropic case
  % Process inputs
  E   = arg1;
  nu  = arg2;
  
  % Compute stiffness properties
  ES  = E .* (1-nu) ./ (1-nu-2*nu.^2);
  nuS = nu./(1-nu);
  
  % Prepare outputs
  out1= ES;
  out2= nuS;
  
%-------------------------------------------------------------------------------
elseif (nargin<=4) % Transverse isotropic
  
  % Process inputs
  Ez   = arg1;
  Et   = arg2;
  nuzt = arg3;
  nut  = arg4;
  
  % nutz set by symmetry of compliance tensor
  nutz    = nuzt .* Et ./ Ez;
  
  % Compute stiffness properties
  ESz   = Ez .* (1-nut) ./ (1-nut-2*nutz.*nuzt);
  ESt   = Et .* (1-nutz.*nuzt) ./ (1-nut-2*nutz.*nuzt) ./ (1+nut);
  nuSt  = (nut+nutz.*nuzt) ./ (1-nutz.*nuzt);
  nuStz = nutz ./ (1-nut);
  nuSzt = nuzt .* (1+nut) ./ (1-nutz.*nuzt);
  
  % Prepare outputs
  out1 = ESz;
  out2 = ESt;
  out3 = nuSzt;
  out4 = nuStz;
  out5 = nuSt;
  
end

end