function [mStiff] = StiffnessMatrix(arg1,arg2,arg3,arg4,arg5)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/15
%
% Finds the stiffness matrix, which multiplies the strain vector to produce
% the stress vector.
% [sigR;sigTheta;sigZ] = mStiff*[epsR;epsTheta;epsZ]
% and when multi-dimensional:
% [sigR(i);sigTheta(i);sigZ(i)] = mStiff(:,:,i)*[epsR(i);epsTheta(i);epsZ(i)]
%
%-------------------------------------------------------------------------------
% Usage:
% [mStiff] = StiffnessMatrix(ES,nuS)
% [mStiff] = StiffnessMatrix(ESz,ESt,nuSzt,nuStz,nuSt)
%-------------------------------------------------------------------------------
% Inputs:
% Set 1:
% ES(1,:)          Isotropic Young's modulus in stiffness form (Pa)
% nuS(1,:)         Isotropic Poisson's ratio in stiffness form
% Set 2:
% ESz(1,:)         Axial Young's modulus in stiffness form (Pa)
% ESt(1,:)         Transverse Young's modulus in stiffness form (Pa)
% nuSzt(1,:)       Poisson's ratio in the axial-transverse direction in 
%                  stiffness form
% nuStz(1,:)       Poisson's ratio in the transverse-axial direction in 
%                  stiffness form
% nuSt(1,:)        Poisson's ratio in the transverse-transverse direction in 
%                  stiffness form
%-------------------------------------------------------------------------------
% Outputs:
% mStiff(3,3,:)    For each combination of properties, the stiffness matrix.
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  [ESz,ESt,nuSzt,nuStz,nuSt] = StiffnessElasticProperties(2e11,1e11,0.3,0.3)
  [mStiff] = StiffnessMatrix(ESz,ESt,nuSzt,nuStz,nuSt)
  mStiff = [];
  return
end
%-------------------------------------------------------------------------------
% Defaults
%-------------------------------------------------------------------------------

% Inputs
if (nargin<=2)
  ESz   = arg1;
  ESt   = arg1;
  nuSzt = arg2;
  nuStz = arg2;
  nuSt  = arg2;
else
  ESz = arg1;
  ESt = arg2;
  nuSzt = arg3;
  nuStz = arg4;
  nuSt = arg5;
end

% Dimension 3 is the matrix working dimension.
ESz   = permute(ESz,[1 3 2]);
ESt   = permute(ESt,[1 3 2]);
nuSzt = permute(nuSzt,[1 3 2]);
nuStz = permute(nuStz,[1 3 2]);
nuSt  = permute(nuSt,[1 3 2]);

mStiff = [ESt.*[ones(size(ESz)) nuSt nuSzt];
          ESt.*[nuSt ones(size(ESz)) nuSzt];
          ESz.*[nuStz nuStz ones(size(ESz))]];

end