function [epsR,epsTheta,epsZ,u] = StrainLameVector(AVec,r,strainLayers)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/15
%
% Finds the strain and radial displacement at several radial points. Computes
% these quantities from the vector of Lame thick cylinder parameters, including
% A and B such that 
% u = A*r + B/r
% The other elements of the vector include axial strain and the in/homogeneous
% coordinate.
%
%-------------------------------------------------------------------------------
% Usage:
% [epsR,epsTheta,epsZ,u] = StrainLameVector(AVec,r,strainLayers)
%-------------------------------------------------------------------------------
% Inputs:
% AVec(:,:)        The vector of Lame thick cylinder parameters. Dimension 2:
%                  different points at which to evaluate strain and displacement
%                  Dimension 1: The vector. Elements 1 and 2 must be A,B. Other
%                  elements consist of axial strains and in/homogeneous 
%                  coordinate, 1.0.
% r(1,:)           The radii of the different points at which to evaluate
%                  strain and displacement. (m)
% strainLayers(1,:)
%                  For each point, the element number of the Lame vector 
%                  that corresponds to the axial strain of that layer.
%                  For example, if every layer were under the axial 
%                  strain given in the 4th element of the solution vector, 
%                  strainLayers would be [4 4 4 4 ...]. Default is all in 
%                  element 3.
%-------------------------------------------------------------------------------
% Outputs:
% epsR(1,:)        Radial strain
% epsTheta(1,:)    Azimuthal strain
% epsZ(1,:)        Axial strain
% u(1,:)           Radial displacement (m)
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  [epsR,epsTheta,epsZ,u] = StrainLameVector([1;1;1],1)
  epsR = [];
  return
end
%-------------------------------------------------------------------------------
% Defaults
if (nargin<=2)
  strainLayers = [];
end
if (isempty(strainLayers))
  strainLayers = 3*ones(1,numel(r));
end
%-------------------------------------------------------------------------------

% Strains
epsR     = AVec(1,:) - AVec(2,:)./r.^2;
epsTheta = AVec(1,:) + AVec(2,:)./r.^2;
epsZ     = AVec(strainLayers+(0:numel(r)-1)*size(AVec,1));

% Displacement
u = AVec(1,:).*r + AVec(2,:)./r;

% NaNs at r=0, B=0
zeroInds = 1:numel(r);
zeroInds = zeroInds(AVec(2,:)==0);
epsR(zeroInds) = AVec(1,zeroInds);
epsTheta(zeroInds) = AVec(1,zeroInds);
u(zeroInds) = AVec(1,zeroInds).*r(zeroInds);

end