function [f,fIntA,fIntB] = UniformBodyForceInts(r,layer,rLayers,fLayers)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
% Layered Axisymmetric Plane Strain Model and Solver Toolbox                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Charles Swanson, cswanson@pppl.gov
% 2022/02/09
%
% Finds the relevant integrals of the volumetric radial body force to transform
% the thick cylinder Lame parameters A,B between two different radii
% Assumes each layer has a uniform volumetric body force
% Specifically, used to compute A(r),B(r) given A,B at the next layer transition
% outward of r.
%
% Described in Section 13 of the derivation doc
%
%-------------------------------------------------------------------------------
% Usage:
% [f,fIntA,fIntB] = UniformBodyForceInts(r,layer,rLayers,fLayers)
%-------------------------------------------------------------------------------
% Inputs:
% r(1,:)           Radii of points at which to evaluate the force integrals (m)
% layer(1,:)       The layer number of each r(1,:) point
% rLayers(1,N+1)   The inner radii of each layer. rLayers(N+1) is the outer 
%                  radius of the outermost layer, layer N.
% fLayers(1,N)     The uniform body force of each layer (N/m^3)
%-------------------------------------------------------------------------------
% Outputs:
% f                Volumetric radial body force at each r point (N/m^3)
% fIntA            Integral of f between r and the next outer rLayer point. 
%                  This is used to transform the Lame A parameter at the outer
%                  point to the A parameter at r. (N/m^2)
% fIntB            Integral of f*r^2 between r and the next outer rLayer point. 
%                  This is used to transform the Lame B parameter at the outer
%                  point to the B parameter at r. (N/m)
%-------------------------------------------------------------------------------
% Demo function
if (nargin==0)
  dr = 1e-2;
  rLayers = [0.25 0.5 0.75];
  fLayers = [2 1];
  r = rLayers(1):dr:rLayers(end);
  layer(r<rLayers(2))  = 1;
  layer(r>=rLayers(2)) = 2;
  [f,fIntA,fIntB] = UniformBodyForceInts(r,layer,rLayers,fLayers);
  figure();
  subplot(2,2,1);
  plot(r,f);xlabel('r (m)');ylabel('f (N/m^3)');grid on;
  subplot(2,2,2);
  plot(r,fIntA);xlabel('r (m)');ylabel('fIntA (N/m^2)');grid on;
  subplot(2,2,3);
  plot(r,fIntB);xlabel('r (m)');ylabel('fIntB (N)');grid on;
  f = [];
  return
end

%-------------------------------------------------------------------------------


% Uniform body force f
% Body force
fBody = @(rArg,layer) f(layer);
% A integral, integral of f:
fIntA = @(rArg,layer) f(layer).*(r(layer+1)-rArg);
% B integral, integral of fr^2:
fIntB = @(rArg,layer) 1/3*f(layer)*(r(layer+1).^3-rArg.^3);

% Lorentz body force f
% Body force
f = fLayers(layer);
% A integral, integral of f: (including correction for if r(1)==0)
fIntA = fLayers(layer).*(rLayers(layer+1)-r);
% B integral, integral of f*r^2:
fIntB = 1/3*fLayers(layer).*(rLayers(layer+1).^3-r.^3);

end